#!/usr/bin/env python3.9


from functools import singledispatchmethod
from objmodel import *
from tools import remove_dupls


class Atomizer:
    @singledispatchmethod
    def atoms(self, arg):
        raise NotImplementedError(
            f"There is no atoms method for {type(arg)}.")

    @atoms.register
    def _(self, _: Formula) -> List[str]:
        raise Exception("Abstract method: to be implemented in subclasses.")

    @atoms.register
    def _(self, _: TrueFormula) -> List[str]:
        return []

    @atoms.register
    def _(self, _: FalseFormula) -> List[str]:
        return []

    @atoms.register
    def _(self, af: AtomicFormula) -> List[str]:
        return [af.text]

    @atoms.register
    def _(self, pf: ParenFormula) -> List[str]:
        return self.atoms(pf.form)

    @atoms.register
    def _(self, nf: NegFormula) -> List[str]:
        return self.atoms(nf.form)

    @atoms.register
    def _(self, cf: ConjFormula) -> List[str]:
        return remove_dupls(self.atoms(cf.lhs) + self.atoms(cf.rhs))

    @atoms.register
    def _(self, df: DisjFormula) -> List[str]:
        return remove_dupls(self.atoms(df.lhs) + self.atoms(df.rhs))

    @atoms.register
    def _(self, im: ImplFormula) -> List[str]:
        return remove_dupls(self.atoms(im.lhs) + self.atoms(im.rhs))

    @atoms.register
    def _(self, ef: EquivFormula) -> List[str]:
        return remove_dupls(self.atoms(ef.lhs) + self.atoms(ef.rhs))

    @atoms.register
    def _(self, rf: ReleaseFormula) -> List[str]:
        return remove_dupls(self.atoms(rf.lhs) + self.atoms(rf.rhs))

    @atoms.register
    def _(self, uf: UntilFormula) -> List[str]:
        return remove_dupls(self.atoms(uf.lhs) + self.atoms(uf.rhs))

    @atoms.register
    def _(self, wf: WeakUntilFormula) -> List[str]:
        return remove_dupls(self.atoms(wf.lhs) + self.atoms(wf.rhs))

    @atoms.register
    def _(self, bf: BoxFormula) -> List[str]:
        return self.atoms(bf.form)

    @atoms.register
    def _(self, df: DiamFormula) -> List[str]:
        return self.atoms(df.form)
