#!/usr/bin/env python3.9


from functools import singledispatchmethod
from objmodel import *


class Stdizer:
    @singledispatchmethod
    def to_std(self, arg):
        raise NotImplementedError(
            f"There is no to_std method for {type(arg)}.")

    @to_std.register
    def _(self, _: MaudeElem) -> MaudeElem:
        raise Exception("Abstract method: to be implemented in subclasses.")

    @to_std.register
    def _(self, _: Module) -> Module:
        raise Exception("Abstract method: to be implemented in subclasses.")

    @to_std.register
    def _(self, fm: FModule) -> FModule:
        return fm

    @to_std.register
    def _(self, t: Theory) -> Theory:
        return Theory(t.name, self.to_std(t.body))

    @to_std.register
    def _(self, v: View) -> View:
        return View(v.name, v.mod_from, v.mod_to, self.to_std(v.body))

    @to_std.register
    def _(self, tb: ThBody) -> ThBody:
        instrs = []
        for i in tb.instrs:
            if not isinstance(i, PptDecl) and not isinstance(i, AGStat):
                instrs.append(i)
        return ThBody(*instrs)

    @to_std.register
    def _(self, vb: ViewBody) -> ViewBody:
        return ViewBody(*[self.to_std(m) for m in vb.maps])

    @to_std.register
    def _(self, m: Mapping) -> Mapping:
        return m

    @to_std.register
    def _(self, pm: PptMapping) -> OpMapping:
        return OpMapping(pm.lhs, pm.rhs)
