#!/usr/bin/env python3.9


from importlib import resources
import pytest
import lark
import terminals
from bodyprocessor import BodyProcessor
from other import astToObjModel
from tools import larkify


@pytest.fixture
def bpr() -> BodyProcessor:
    body_text =\
        """
        sort S .
        sort T .
        op s : -> S .
        op t : -> T .
        vars X1 X2 : S .
        var Y : T .
        rl X1 => s .
        crl Y => t if Y = t . 
        """
    larkText = resources.read_text("grammars", "cmaude.lark")
    cMaudeGrammarText = terminals.terminal_defs() + larkText
    cMaudeGrammar = lark.Lark(larkify(cMaudeGrammarText), start="mod_body")
    bodyAst = cMaudeGrammar.parse(body_text)
    body = astToObjModel(bodyAst)
    return BodyProcessor(body)


def test_eqs(bpr):
    assert bpr.eqs == []


def test_non_eg_rules(bpr):
    assert len(bpr.all_non_eg_rules) == 2


def test_decl_sorts(bpr):
    assert bpr.get_declared_sorts() == ["S", "T"]


def test_decl_names(bpr):
    assert sorted(bpr.get_declared_names())\
           == ["S", "T", "X1", "X2", "Y", "s", "t"]
