#!/usr/bin/env python3.9


from importlib import resources
import re
import lark
import terminals
from inliner import Inliner
from other import astToObjModel
from tools import larkify


def simple(s: str) -> str:
    s1 = s.replace('\t', ' ')
    s2 = s1.replace('\n', ' ')
    s3 = re.sub(' +', ' ', s2)
    s4 = s3.strip()
    return s4


# noinspection PyUnresolvedReferences
def test_ceq():
    orig = "ceq X = Y if X = 1 ."
    res = "ceq X:S = Y:T if X:S = 1 ."
    inlr = Inliner({"X": "S", "Y": "T"})
    assert res == inlr._Inliner__inline_vars(orig)


def test_body():
    """Beware that this test would fail if several sorts are declared in the
    same line (`sorts S T .`), because the `str` applied to a body generates
    them each in a line."""
    orig =\
        """
        sort S .
        sort T .
        op s : -> S .
        op t : -> T .
        vars X1 X2 : S .
        var Y : T .
        rl X1 => s .
        crl Y => t if Y = t . 
        """
    res =\
        """
        sort S .
        sort T .
        ops s : -> S .
        ops t : -> T .
        rl X1:S => s .
        crl Y:T => t if Y:T = t . 
        """
    larkText = resources.read_text("grammars", "cmaude.lark")
    cMaudeGrammarText = terminals.terminal_defs() + larkText
    cMaudeGrammar = lark.Lark(larkify(cMaudeGrammarText), start="mod_body")
    bodyAst = cMaudeGrammar.parse(orig)
    body = astToObjModel(bodyAst)
    assert simple(res) == simple(str(Inliner.inline_vars_in_body(body)))
