#!/usr/bin/env python3.9


import re
from typing import List, Callable, Any


def remove_dupls(ss: List[Any], f: Callable = lambda x: x) -> List[Any]:
    res = []
    for s in ss:
        if f(s) not in [f(r) for r in res]:
            res.append(s)
    return res


def buildName(*ss: str) -> str:
    res = "<"
    for s in ss:
        if s.startswith("<") and s.endswith(">"):
            res += s[1:-1] + "|"
        else:
            res += s + "|"
    return res[:-1] + ">"


def buildPptSortName(s: str) -> str:
    return buildName("Ppt", s)


def freshVarName() -> str:
    try:
        freshVarName.counter += 1
    except AttributeError:
        freshVarName.counter = 1
    # This seems to be the pythonic way to emulate static (or persistent) vars
    return "Var" + str(freshVarName.counter)


def symbolToClassName(symbol: str) -> str:
    # This produces the name of the class from the name of the syntactic entity.
    # From `abc_def___ghi` it produces `AbcDef`.
    res = symbol
    i = res.find("___")
    if i != -1:
        res = res[0:i]
    res = res[0].upper() + res[1:]
    res = re.sub("_(.)", lambda m: m.groups()[0].upper(), res)
    return res


def dollarToName(s: str) -> str:
    return re.sub(r"([A-Z0-9\-]+)\$([^\s]+)", r"<\1|\2>", s)


def compStgName() -> str:
    return "<compStg>"


def makeCompStg(ss: List[str]) -> str:
    return compStgName() + "(" + ", ".join(ss) + ")"


def split_dollarized_id(s: str) -> List[str]:
    return s.split("$")


def larkify(gr: str) -> str:
    # make name of entity
    gr1 = re.sub(r"(\w+)\((\w+)\): ", r"\2___\1: ", gr,
                 re.MULTILINE)
    # remove names for attributes
    gr2 = re.sub(r"(?<=\W)\w+:(\w+)(?=\W)", r"\1", gr1)
    # remove formats
    gr3 = re.sub(r"\s[nsxi](\s|$)", r"\1", gr2)
    # join lines
    gr4 = re.sub(r" +\\ *\n *", " ", gr3)
    # expand Python-style joins
    gr5 = re.sub(r'("[^"]+")([+*])([A-Za-z0-9_]+)', r'\3 (\1 \3)\2', gr4)
    return gr5


def joinlines(gr: str) -> str:
    return re.sub(r" +\\ *\n *", " ", gr)


def indent(lines: str, width: int = 4):
    return "".join([" "*width + line for line in lines.splitlines(True)])
